"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const token_1 = require("@project-serum/anchor/dist/cjs/spl/token");
const web3_js_1 = require("@solana/web3.js");
const dotenv = __importStar(require("dotenv"));
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../client");
const constants_1 = require("../constants");
const I80F48_1 = require("../numbers/I80F48");
const utils_1 = require("../utils");
dotenv.config();
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const PAYER_KEYPAIR = process.env.PAYER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
async function main() {
    var _a;
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(PAYER_KEYPAIR, 'utf-8'))));
    const adminWallet = new anchor_1.Wallet(admin);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = client_1.MangoClient.connect(adminProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], { idsSource: 'get-program-accounts' });
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`Group ${group.publicKey.toBase58()}`);
    const banks = Array.from(group.banksMapByMint.values()).flat();
    const banksMapUsingTokenIndex = new Map(banks.map((bank) => {
        bank.indexedDepositsByMangoAccounts = (0, I80F48_1.ZERO_I80F48)();
        bank.indexedBorrowsByMangoAccounts = (0, I80F48_1.ZERO_I80F48)();
        return [bank.tokenIndex, bank];
    }));
    const mangoAccounts = await client.getAllMangoAccounts(group);
    mangoAccounts.map((mangoAccount) => mangoAccount.tokensActive().forEach((token) => {
        const bank = banksMapUsingTokenIndex.get(token.tokenIndex);
        if (token.indexedPosition.isPos()) {
            bank.indexedDepositsByMangoAccounts = bank.indexedDepositsByMangoAccounts.add(token.indexedPosition.mul(banksMapUsingTokenIndex.get(token.tokenIndex).depositIndex));
        }
        if (token.indexedPosition.isNeg()) {
            bank.indexedBorrowsByMangoAccounts = bank.indexedBorrowsByMangoAccounts.add(token.indexedPosition
                .abs()
                .mul(banksMapUsingTokenIndex.get(token.tokenIndex).borrowIndex));
        }
    }));
    for (const bank of await Array.from(banksMapUsingTokenIndex.values()).sort((a, b) => a.tokenIndex - b.tokenIndex)) {
        const vault = I80F48_1.I80F48.fromNumber((0, token_1.coder)()
            .accounts.decode('token', (await client.program.provider.connection.getAccountInfo(bank.vault))
            .data)
            .amount.toNumber());
        const error = vault.sub(bank.indexedDepositsByMangoAccounts
            .sub(bank.indexedBorrowsByMangoAccounts)
            .add(bank.collectedFeesNative)
            .add(bank.dust));
        let res = `${bank.name}`;
        res =
            res +
                `\n ${'tokenIndex'.padEnd(40)} ${bank.tokenIndex}` +
                `\n ${'bank'.padEnd(40)} ${bank.publicKey}` +
                `\n ${'vault'.padEnd(40)} ${bank.vault}` +
                `\n ${'oracle'.padEnd(40)} ${bank.oracle}` +
                `\n ${'mint'.padEnd(40)} ${bank.mint}` +
                `\n ${'price'.padEnd(40)} ${(_a = bank.price) === null || _a === void 0 ? void 0 : _a.toNumber()}` +
                `\n ${'uiPrice'.padEnd(40)} ${bank.uiPrice}` +
                `\n ${'error'.padEnd(40)} ${error}` +
                `\n ${'collectedFeesNative'.padEnd(40)} ${bank.collectedFeesNative}` +
                `\n ${'dust'.padEnd(40)} ${bank.dust}` +
                `\n ${'vault balance'.padEnd(40)} ${(0, utils_1.toUiDecimals)(vault, bank.mintDecimals)}, ${vault} native` +
                `\n ${'deposits'.padEnd(40)} ${bank.indexedDeposits.mul(bank.depositIndex)}` +
                `\n ${'deposits (sum over all mango accounts)'.padEnd(40)} ${bank.indexedDepositsByMangoAccounts}` +
                `\n ${'borrows'.padEnd(40)} ${bank.indexedBorrows.mul(bank.borrowIndex)}` +
                `\n ${'borrows (sum over all mango accounts)'.padEnd(40)} ${bank.indexedBorrowsByMangoAccounts}` +
                `\n ${'avgUtilization since last rate update'.padEnd(40)} ${(100 * bank.avgUtilization.toNumber()).toFixed(1)}%` +
                `\n ${'rate parameters'.padEnd(40)} ${(100 * bank.rate0.toNumber()).toFixed()}% @ ${(100 * bank.util0.toNumber()).toFixed()}% util, ${(100 * bank.rate1.toNumber()).toFixed()}% @${(100 * bank.util1.toNumber()).toFixed()}% util, ${(100 * bank.maxRate.toNumber()).toFixed()}% @ 100% util` +
                `\n ${'depositRate'.padEnd(40)} ${(100 * bank.getDepositRate().toNumber()).toFixed(2)}%` +
                `\n ${'borrowRate'.padEnd(40)} ${(100 * bank.getBorrowRate().toNumber()).toFixed(2)}%` +
                `\n ${'last index update'.padEnd(40)} ${new Date(1000 * bank.indexLastUpdated.toNumber())}` +
                `\n ${'last rates update'.padEnd(40)} ${new Date(1000 * bank.bankRateLastUpdated.toNumber())}` +
                `\n ${'net borrows in window'.padEnd(40)} ${bank.netBorrowsInWindow.toNumber()} / ${bank.netBorrowLimitPerWindowQuote.toNumber()}`;
        console.log(`${res}`);
    }
    process.exit();
}
try {
    main();
}
catch (error) {
    console.log(error);
}
//# sourceMappingURL=debug-banks.js.map