"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEVNET_MINTS = void 0;
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../client");
const constants_1 = require("../constants");
//
// example script to close accounts - banks, markets, group etc. which require admin to be the signer
//
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
exports.DEVNET_MINTS = new Map([
    ['USDC', '8FRFC6MoGGkMFQwngccyu69VnYbzykGeez7ignHVAFSN'], // use devnet usdc
]);
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection('https://mango.devnet.rpcpool.com', options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
    const adminWallet = new anchor_1.Wallet(admin);
    console.log(`Admin ${adminWallet.publicKey.toBase58()}`);
    const adminProvider = new anchor_1.AnchorProvider(connection, adminWallet, options);
    const client = await client_1.MangoClient.connect(adminProvider, 'devnet', constants_1.MANGO_V4_ID['devnet'], {
        idsSource: 'get-program-accounts',
    });
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(`Group ${group.publicKey}`);
    let sig;
    // close stub oracle
    const usdcDevnetMint = new web3_js_1.PublicKey(exports.DEVNET_MINTS.get('USDC'));
    const usdcDevnetOracle = (await client.getStubOracle(group, usdcDevnetMint))[0];
    sig = await client.stubOracleClose(group, usdcDevnetOracle.publicKey);
    console.log(`Closed USDC stub oracle, sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    // close all bank
    group.consoleLogBanks();
    // deregister all serum markets
    for (const market of group.serum3MarketsMapByExternal.values()) {
        sig = await client.serum3deregisterMarket(group, market.serumMarketExternal);
        console.log(`Deregistered serum market ${market.name}, sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    }
    // close all perp markets
    for (const market of group.perpMarketsMapByMarketIndex.values()) {
        sig = await client.perpCloseMarket(group, market.perpMarketIndex);
        console.log(`Closed perp market ${market.name}, sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    }
    // finally, close the group
    sig = await client.groupClose(group);
    console.log(`Closed group, sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
    process.exit();
}
main();
//# sourceMappingURL=devnet-admin-close.js.map