"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const serum3_1 = require("../accounts/serum3");
const client_1 = require("../client");
const constants_1 = require("../constants");
//
// (untested?) script which closes a mango account cleanly, first closes all positions, withdraws all tokens and then closes it
//
async function closeUserAccount(userKeypairFile) {
    var _a, _b, _c, _d;
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(process.env.MB_CLUSTER_URL, options);
    // user
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(userKeypairFile, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta']);
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    // admin
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.MB_PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Admin ${admin.publicKey.toBase58()}`);
    // fetch group
    const group = await client.getGroupForCreator(admin.publicKey, 2);
    console.log(`Found group ${group.publicKey.toBase58()}`);
    // account
    const mangoAccount = (await client.getMangoAccountsForOwner(group, user.publicKey))[0];
    console.log(`...found mangoAccount ${mangoAccount.publicKey}`);
    console.log(mangoAccount.toString(group));
    try {
        // cancel serum3 accounts, closing might require cancelling orders and settling
        for (const serum3Account of mangoAccount.serum3Active()) {
            let orders = await mangoAccount.loadSerum3OpenOrdersForMarket(client, group, (_a = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _a === void 0 ? void 0 : _a.serumMarketExternal);
            for (const order of orders) {
                console.log(` - Order orderId ${order.orderId}, ${order.side}, ${order.price}, ${order.size}`);
                console.log(` - Cancelling order with ${order.orderId}`);
                await client.serum3CancelOrder(group, mangoAccount, (_b = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _b === void 0 ? void 0 : _b.serumMarketExternal, order.side === 'buy' ? serum3_1.Serum3Side.bid : serum3_1.Serum3Side.ask, order.orderId);
            }
            await client.serum3SettleFunds(group, mangoAccount, (_c = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _c === void 0 ? void 0 : _c.serumMarketExternal);
            await client.serum3CloseOpenOrders(group, mangoAccount, (_d = group.serum3MarketsMapByMarketIndex.get(serum3Account.marketIndex)) === null || _d === void 0 ? void 0 : _d.serumMarketExternal);
        }
        // we closed a serum account, this changes the health accounts we are passing in for future ixs
        await mangoAccount.reload(client);
        // withdraw all tokens
        for (const token of mangoAccount.tokensActive()) {
            const native = token.balance(group.getFirstBankByTokenIndex(token.tokenIndex));
            console.log(`token native ${native} ${group.getFirstBankByTokenIndex(token.tokenIndex).name}`);
            if (native.toNumber() < 1) {
                continue;
            }
            const withdrawIx = await client.tokenWithdrawNativeIx(group, mangoAccount, group.getFirstBankByTokenIndex(token.tokenIndex).mint, new anchor_1.BN(token
                .balance(group.getFirstBankByTokenIndex(token.tokenIndex))
                .toNumber()), false);
            await this.sendAndConfirmTransaction([...withdrawIx], group.addressLookupTablesList);
        }
    }
    catch (error) {
        console.log(error);
    }
    await mangoAccount.reload(client);
    console.log(`...mangoAccount ${mangoAccount.publicKey}`);
    console.log(mangoAccount.toString());
    console.log(`Close mango account...`);
    const res = await client.closeMangoAccount(group, mangoAccount);
    process.exit();
}
async function main() {
    // await closeUserAccount(process.env.MB_USER_KEYPAIR!);
    await closeUserAccount(process.env.MB_USER2_KEYPAIR);
}
main();
//# sourceMappingURL=mb-close-account.js.map