"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const client_1 = require("../client");
const constants_1 = require("../constants");
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
const MANGO_ACCOUNT_PK = process.env.MANGO_ACCOUNT_PK;
const { MB_CLUSTER_URL, MB_PAYER_KEYPAIR, MB_USER_KEYPAIR, MB_USER4_KEYPAIR } = process.env;
async function buildUserClient(userKeypair) {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(MB_CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(userKeypair, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'mainnet-beta', constants_1.MANGO_V4_ID['mainnet-beta']);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(MB_PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Admin ${admin.publicKey.toBase58()}`);
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    return [client, group, user];
}
async function forceCloseUserAccount() {
    const result = await buildUserClient(MB_PAYER_KEYPAIR);
    const client = result[0];
    const group = result[1];
    const mangoAccount = await client.getMangoAccount(new web3_js_1.PublicKey(MANGO_ACCOUNT_PK));
    await client.closeMangoAccount(group, mangoAccount, true);
    process.exit();
}
async function main() {
    await forceCloseUserAccount();
}
main();
//# sourceMappingURL=mb-force-close-account.js.map