"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const mangoAccount_1 = require("../accounts/mangoAccount");
const index_1 = require("../index");
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(process.env.MB_CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.MB_PAYER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await index_1.MangoClient.connect(userProvider, 'mainnet-beta', index_1.MANGO_V4_ID['mainnet-beta']);
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.MB_PAYER_KEYPAIR, 'utf-8'))));
    console.log(`Admin ${admin.publicKey.toBase58()}`);
    const group = await client.getGroupForCreator(admin.publicKey, 2);
    console.log(`${group.toString()}`);
    // create + fetch account
    console.log(`Creating mangoaccount...`);
    const mangoAccount = await client.getOrCreateMangoAccount(group);
    console.log(`...created/found mangoAccount ${mangoAccount.publicKey}`);
    console.log(mangoAccount.toString(group));
    if (true) {
        console.log(`...depositing 0.0001 USDC`);
        await client.tokenDeposit(group, mangoAccount, group.banksMapByName.get('USDC')[0].mint, 10);
        await mangoAccount.reload(client);
        console.log(`...depositing 0.001 SOL`);
        await client.tokenDeposit(group, mangoAccount, group.banksMapByName.get('SOL')[0].mint, 1);
        await mangoAccount.reload(client);
    }
    await mangoAccount.reload(client);
    console.log('mangoAccount.getEquity() ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.getEquity(group).toNumber()));
    console.log('mangoAccount.getHealth(HealthType.init) ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.getHealth(group, mangoAccount_1.HealthType.init).toNumber()));
    console.log('mangoAccount.getHealthRatio(HealthType.init) ' +
        mangoAccount.getHealthRatio(group, mangoAccount_1.HealthType.init).toNumber());
    console.log('mangoAccount.getCollateralValue() ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.getCollateralValue(group).toNumber()));
    console.log('mangoAccount.getAssetsVal() ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.getAssetsValue(group, mangoAccount_1.HealthType.init).toNumber()));
    console.log('mangoAccount.getLiabsVal() ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.getLiabsValue(group, mangoAccount_1.HealthType.init).toNumber()));
    console.log("mangoAccount.getMaxWithdrawWithBorrowForToken(group, 'SOL') " +
        (0, index_1.toUiDecimalsForQuote)((await mangoAccount.getMaxWithdrawWithBorrowForToken(group, group.banksMapByName.get('SOL')[0].mint)).toNumber()));
    console.log("mangoAccount.getMaxSourceForTokenSwap(group, 'USDC', 'BTC') " +
        (await mangoAccount.getMaxSourceUiForTokenSwap(group, group.banksMapByName.get('USDC')[0].mint, group.banksMapByName.get('BTC')[0].mint, 0.94)));
    console.log('mangoAccount.simHealthWithTokenPositionChanges ' +
        (0, index_1.toUiDecimalsForQuote)(mangoAccount.simHealthRatioWithTokenPositionUiChanges(group, [
            {
                mintPk: group.banksMapByName.get('USDC')[0].mint,
                uiTokenAmount: -20000 *
                    Math.pow(10, group.banksMapByName.get('BTC')[0].mintDecimals),
            },
            {
                mintPk: group.banksMapByName.get('BTC')[0].mint,
                uiTokenAmount: 1 *
                    Math.pow(10, group.banksMapByName.get('BTC')[0].mintDecimals),
            },
        ])));
    process.exit();
}
main();
//# sourceMappingURL=mb-user.js.map