"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@project-serum/anchor");
const web3_js_1 = require("@solana/web3.js");
const client_1 = require("../../client");
const constants_1 = require("../../constants");
// For easy switching between mainnet and devnet, default is mainnet
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const MANGO_ACCOUNT_PK = process.env.MANGO_ACCOUNT_PK || '';
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    // Throwaway keypair
    const user = new web3_js_1.Keypair();
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
    });
    // Load mango account
    let mangoAccount = await client.getMangoAccount(new web3_js_1.PublicKey(MANGO_ACCOUNT_PK));
    await mangoAccount.reload(client);
    // Load group for mango account
    const group = await client.getGroup(mangoAccount.group);
    await group.reloadAll(client);
    // Log OB
    while (true) {
        await new Promise((r) => setTimeout(r, 2000));
        console.clear();
        await group.reloadAll(client);
        const btcPerpMarket = group.getPerpMarketByName('BTC-PERP');
        console.log(`  perpMarket.uiPrice ${btcPerpMarket.uiPrice}`);
        console.log(``);
        console.log(await btcPerpMarket.logOb(client));
        console.log(``);
        const mngoPerpMarket = group.getPerpMarketByName('MNGO-PERP');
        console.log(`  perpMarket.uiPrice ${mngoPerpMarket.uiPrice}`);
        console.log(``);
        console.log(await mngoPerpMarket.logOb(client));
        console.log(``);
    }
}
main();
//# sourceMappingURL=log-perp-ob.js.map